<?php
/**
 * 
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Block;

use Magento\Framework\View\Element\Template\Context;
use Magento\Framework\Phrase;
use Magento\Payment\Block\ConfigurableInfo;
use Magento\Payment\Gateway\ConfigInterface;
use Risecommerce\Elavon\Helper\Data;

/**
 * Class Info
 */
class Info extends ConfigurableInfo
{
    /**
     * @var Data
     */
    protected $customHelper;

    /**
     * @param Context $context
     * @param ConfigInterface $config
     * @param Data $customHelper
     * @param array $data
     */
    public function __construct(
        Context $context,
        ConfigInterface $config,
        Data $customHelper,
        array $data = []
    ) {
        parent::__construct($context, $config, $data);
        $this->customHelper = $customHelper;
    }

    protected $_avsCode =  [
        'A' => 'Address matches - ZIP Code does not match', 'B' => 'Street address match, Postal code in wrong format (international issuer)',
        'C' => 'Street address and postal code in wrong formats', 'D' => 'Street address and postal code match (international issuer)',
        'E' => 'AVS Error', 'F' => 'Address does compare and five-digit ZIP code does compare (UK only)',
        'G' => 'Service not supported by non-US issuer', 'I' => 'Address information not verified by international issuer',
        'M' => 'Street Address and Postal code match (international issuer)', 'N' => 'No Match on Address (Street) or ZIP',
        'O' => 'No Response sent', 'P' => 'Postal codes match, Street address not verified due to incompatible formats',
        'R' => 'Retry, System unavailable or Timed out', 'S' => 'Service not supported by issuer',
        'U' => 'Address information is unavailable', 'W' => '9-digit ZIP matches, Address (Street) does not match',
        'X' => 'Exact AVS Match', 'Y' => 'Address (Street) and 5-digit ZIP match','Z' => '5-digit ZIP matches, Address (Street) does not match'
    ];

    protected $_cvvCode =  [
        'M' => 'CVV2/CVC2 Match', 'N' => 'CVV2/CVC2 No match', 'P' => 'Not processed',
        'S' => 'Issuer indicates that the CVV2/CVC2 data should be present on the card, but the merchant has indicated that the CVV2/CVC2 data is not resent on the card',
        'U' => 'Issuer has not certified for CVV2/CVC2 or Issuer has not provided Visa with the CVV2/CVC2 encryption keys'
    ];

    /**
     * Returns label
     *
     * @param string $field
     * @return Phrase
     */
    protected function getLabel($field)
    {
        switch ($field) {
            case 'cc_num':
                return __('Card Number');
            case 'cc_type':
                return __('Card Type');
            case 'order_tra_id':
                return __('Transaction ID');
            case 'last_transaction_id':
                return __('Last Transaction ID');
            case 'ssl_approval_code':
                return __('Authorization Code');
            case 'avs_result':
                return __('AVS Code');
            case 'cvd_result':
                return __('CVV Response');
            case 'payment_status':
                return __('Payment Status');
            default:
                return parent::getLabel($field);
        }
    }

    /**
     * Returns value view
     *
     * @param string $field
     * @param string $value
     * @return string | Phrase
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    protected function getValueView($field, $value)
    {
        if ($field == 'cc_type') {
            if ($this->customHelper->getCcTypeNameByCode($value)) {
                $value = $this->customHelper->getCcTypeNameByCode($value);
            }
        }

        if ($field == 'avs_res_id') {
            if (isset($this->_avsCode[trim($value)])) {
                $value = $this->_avsCode[trim($value)];
            }
        }

        if ($field == 'cvd_result') {
            if (isset($this->_cvvCode[trim($value)])) {
                $value = $this->_cvvCode[trim($value)];
            }
        }

        if ($field == 'risk_score') {
            if ($value == 0) {
                $value = (string) '0.0';
            }
        }

        return $value;
    }
}
