<?php
/**
 * Elavon Hosted Payment Module.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Controller\Hosted;

use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Exception\LocalizedException;

class Approved extends AbstractExpress
{
    /** @var string */
    protected $_checkoutType = \Risecommerce\Elavon\Model\HostedPage\Checkout::class;

    /** @var \Risecommerce\Elavon\Helper\Data $customHelper */
    protected $customHelper;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     * @param \Magento\Paypal\Model\Express\Checkout\Factory $checkoutFactory
     * @param \Magento\Framework\Session\Generic $paypalSession
     * @param \Magento\Framework\Url\Helper\Data $urlHelper
     * @param \Magento\Customer\Model\Url $customerUrl
     * @param \Risecommerce\Elavon\Helper\Data $customHelper
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Paypal\Model\Express\Checkout\Factory $checkoutFactory,
        \Magento\Framework\Session\Generic $paypalSession,
        \Magento\Framework\Url\Helper\Data $urlHelper,
        \Magento\Customer\Model\Url $customerUrl,
        \Risecommerce\Elavon\Helper\Data $customHelper
    ) {
        $this->customHelper = $customHelper;
        parent::__construct(
            $context,
            $customerSession,
            $checkoutSession,
            $orderFactory,
            $checkoutFactory,
            $paypalSession,
            $urlHelper,
            $customerUrl
        );
    }

    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);

        $responseData = $this->getRequest()->getParams();
        if ($responseData['ssl_result'] == '0') {
            $transactionId = '';
            if (!empty($responseData['ssl_txn_id'])) {
                $transactionId = $responseData['ssl_txn_id'];
            }
            try {
                $this->_initCheckout();

                $quote = $this->_getQuote();
                $quoteId = $quote->getId();
                $quote->getPayment()->setAdditionalInformation('order_tra_id', $transactionId);

                $this->_checkout->place(null);

                // prepare session to success or cancellation page
                $this->_getCheckoutSession()->clearHelperData();

                // "last successful quote"
                $this->_getCheckoutSession()->setLastQuoteId($quoteId)->setLastSuccessQuoteId($quoteId);

                // an order may be created
                $order = $this->_checkout->getOrder();
                if ($order) {
                    $payment = $order->getPayment();

                    $payment->setAdditionalInformation('order_tra_id', $transactionId);
                    if (!empty($responseData['ssl_card_number'])) {
                        $ccLast4Digits = sprintf('xxxx-%s', substr($responseData['ssl_card_number'], -4));
                        $payment->setAdditionalInformation('cc_number', $ccLast4Digits);
                    }
                    if (!empty($responseData['card'])) {
                        $payment->setAdditionalInformation('cc_type', $responseData['card']);
                    }
                    if (!empty($responseData['ssl_approval_code'])) {
                        $payment->setAdditionalInformation('ssl_approval_code', $responseData['ssl_approval_code']);
                    }
                    if (!empty($responseData['ssl_avs_response'])) {
                        $payment->setAdditionalInformation('avs_result', $responseData['ssl_avs_response']);
                    }
                    if (!empty($responseData['ssl_cvv2_response'])) {
                        $payment->setAdditionalInformation('cvd_result', $responseData['ssl_cvv2_response']);
                    }
                    $paymentStatus = $responseData['ssl_transaction_type'] == 'SALE' ? 'Captured' : 'Authorize Only';
                    $payment->setAdditionalInformation('payment_status', $paymentStatus);
                    $payment->save();

                    $this->_getCheckoutSession()->setLastOrderId($order->getId())
                        ->setLastRealOrderId($order->getIncrementId())
                        ->setLastOrderStatus($order->getStatus());
                }
                $this->_getSession()->unsQuoteId();
                $this->_redirect('checkout/onepage/success');
                return;
            } catch (LocalizedException $e) {
                $this->messageManager->addExceptionMessage(
                    $e,
                    'We can\'t place the order. Something went wrong, please contact us for more details.'
                );
            }
        } else {
            $this->messageManager->addError(__('We can\'t place the order. Something went wrong, please contact us for more details.'));
        }

        return $resultRedirect->setPath('checkout/cart');
    }
}
