<?php
/**
 * Elavon Payment Module.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Controller\Hosted;

use Magento\Framework\App\Action\HttpGetActionInterface as HttpGetActionInterface;
use Magento\Checkout\Helper\Data;
use Magento\Checkout\Helper\ExpressRedirect;
use Magento\Checkout\Model\Type\Onepage;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Webapi\Exception;
use Magento\Paypal\Model\Express\Checkout;
use Magento\Framework\App\ObjectManager;

class GetToken extends AbstractExpress implements HttpGetActionInterface
{
    /** @var string */
    protected $_configMethod = \Risecommerce\Elavon\Model\PaymentHosted::CODE_HOSTED;

    /** @var string */
    protected $_checkoutType = \Risecommerce\Elavon\Model\HostedPage\Checkout::class;

    /** @var \Magento\Framework\Controller\Result\Redirect */
    protected $resultRedirectFactory;

    /** @var \Risecommerce\Elavon\Helper\Data */
    protected $customHelper;

    /**
     * GetToken constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     * @param Checkout\Factory $checkoutFactory
     * @param \Magento\Framework\Session\Generic $paypalSession
     * @param \Magento\Framework\Url\Helper\Data $urlHelper
     * @param \Magento\Customer\Model\Url $customerUrl
     * @param \Magento\Framework\Controller\Result\Redirect $resultRedirectFactory
     * @param \Risecommerce\Elavon\Helper\Data $customHelper
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Paypal\Model\Express\Checkout\Factory $checkoutFactory,
        \Magento\Framework\Session\Generic $paypalSession,
        \Magento\Framework\Url\Helper\Data $urlHelper,
        \Magento\Customer\Model\Url $customerUrl,
        \Magento\Framework\Controller\Result\Redirect $resultRedirectFactory,
        \Risecommerce\Elavon\Helper\Data $customHelper
    ) {
        $this->customHelper = $customHelper;
        $this->resultRedirectFactory = $resultRedirectFactory;
        parent::__construct(
            $context,
            $customerSession,
            $checkoutSession,
            $orderFactory,
            $checkoutFactory,
            $paypalSession,
            $urlHelper,
            $customerUrl
        );
    }

    /**
     * @inheritdoc
     */
    public function execute()
    {
    }

    /**
     * @return string|null
     * @throws LocalizedException
     */
    protected function getToken()
    {
        $this->_initCheckout();
        $quote = $this->_getQuote();

        /** @var Data $checkoutHelper */
        $checkoutHelper = $this->_objectManager->get(Data::class);
        $quoteCheckoutMethod = $quote->getCheckoutMethod();
        $customerData = $this->_customerSession->getCustomerDataObject();

        if ($customerData->getId()) {
            $this->_checkout->setCustomerWithAddressChange(
                $customerData,
                $quote->getBillingAddress(),
                $quote->getShippingAddress()
            );
        } elseif ((!$quoteCheckoutMethod || $quoteCheckoutMethod !== Onepage::METHOD_REGISTER) && !$checkoutHelper->isAllowedGuestCheckout($quote, $quote->getStoreId())) {
            // This code redreict customer to login page if not logged in. i think not required for us.
            $expressRedirect = $this->_objectManager->get(ExpressRedirect::class);

            $this->messageManager->addNoticeMessage(
                __('To check out, please sign in with your email address.')
            );

            $expressRedirect->redirectLogin($this);
            $this->_customerSession->setBeforeAuthUrl(
                $this->_url->getUrl('*/*/*', ['_current' => true])
            );

            return null;
        }

        return $this->_checkout->start(
            null,
            null,
            null
        );
    }
}
