<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Http;

use Magento\Payment\Gateway\Http\TransferBuilder;
use Magento\Payment\Gateway\Http\TransferFactoryInterface;
use Risecommerce\Elavon\Helper\Data;
use Risecommerce\Elavon\Model\Method\Logger as RcLogger;

/**
 * Class AbstractTransferFactory
 */
abstract class AbstractTransferFactory implements TransferFactoryInterface
{
    protected $generator;

    /**
     * @var TransferBuilder
     */
    protected $transferBuilder;

    /**
     * @var Data
     */
    protected $customHelper;

    /**
     * @var RcLogger
     */
    private $rcLogger;

    /**
     * @param TransferBuilder $transferBuilder
     * @param Data $helper
     * @param RcLogger $rcLogger
     */
    public function __construct(
        TransferBuilder $transferBuilder,
        Data $helper,
        RcLogger $rcLogger
    ) {
        $this->transferBuilder = $transferBuilder;
        $this->customHelper = $helper;
        $this->rcLogger = $rcLogger;
    }

    public function getUrl($orderStoreId = null)
    {
        return $this->customHelper->getPostUrl($orderStoreId);
    }

    public function rcLogger($res)
    {
        if ($this->customHelper->getConfig('payment/risecommerce_elavon_general/debug')) {
            $logKey = !empty($res['ssl_invoice_number']) ? ' of order ' . $res['ssl_invoice_number'] : '';
            $reqRes = ["Request" . $logKey => $res];
            $this->rcLogger->debug($reqRes);
        }
    }

    protected function convertToXml($array, $rootElement = '<txn/>', $xml = null)
    {
        $_xml = $xml;

        // If there is no Root Element then insert root
        if ($_xml === null) {
            $_xml = new \SimpleXMLElement($rootElement !== null ? $rootElement : '<txn/>');
        }

        // Visit all key value pair
        foreach ($array as $k => $v) {
            // If there is nested array then
            if (is_array($v)) {
                if ($k == 'product') {
                    foreach($v as $item) {
                        $this->convertToXml($item, 'product', $_xml->addChild('product'));
                    }
                } else {
                    // Call function for nested array
                    $this->convertToXml($v, $k, $_xml->addChild($k));
                }
            } else {
                $v = !empty($v) ? htmlspecialchars($v) : $v;
                // Simply add child element.
                $_xml->addChild($k, $v);
            }
        }

        return $_xml->asXML();
    }
}
