<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Http\Client;

use Laminas\Http\Exception\RuntimeException;
use Magento\Payment\Gateway\Http\ClientException;
use Magento\Payment\Gateway\Http\ConverterException;
use Magento\Payment\Gateway\Http\ConverterInterface;
use Magento\Payment\Gateway\Http\TransferInterface;
use Risecommerce\Elavon\Model\Method\Logger as RcLogger;

/**
 * Class Zend
 */
class Zend extends \Magento\Payment\Gateway\Http\Client\Zend
{
    /**
     * @var ConverterInterface | null
     */
    private $converter;

    /**
     * @var \Risecommerce\Elavon\Helper\Data
     */
    private $customHelper;

    /**
     * @var RcLogger
     */
    private $rcLogger;

    /**
     * @param ConverterInterface|null $converter
     * @param \Risecommerce\Elavon\Helper\Data $customHelper
     * @param RcLogger $rcLogger
     */
    public function __construct(
        ConverterInterface $converter = null,
        \Risecommerce\Elavon\Helper\Data $customHelper,
        RcLogger $rcLogger
    ) {
        $this->converter = $converter;
        $this->customHelper = $customHelper;
        $this->rcLogger = $rcLogger;
    }

    /**
     * @inheritdoc
     */
    public function placeRequest(TransferInterface $transferObject)
    {
        $resultNew = [];

        /** @var \Laminas\Http\Client $client */
        $client = new \Laminas\Http\Client();
        //$client->setOptions(['timeout' => 45]);

        $client->setMethod($transferObject->getMethod());
        $client->setRawBody($transferObject->getBody());
        $client->setHeaders($transferObject->getHeaders());
        $client->setUri($transferObject->getUri());
        
        try {
            $response = $client->send();
            $new = simplexml_load_string($response->getBody());
            $con = json_encode($new);
            $resultNew = json_decode($con, true);
        } catch (RuntimeException $e) {
            throw new ClientException(__($e->getMessage()));
        } catch (ConverterException $e) {
            throw $e;
        } finally {
            $this->rcLogger($resultNew);
        }

        return $resultNew;
    }

    public function rcLogger($res)
    {
        if ($this->customHelper->getConfig('payment/risecommerce_elavon_general/debug')) {
            $logKey = !empty($res['ssl_invoice_number']) ? ' of order ' . $res['ssl_invoice_number'] : '';
            $reqRes = ["Response" . $logKey => $res];
            $this->rcLogger->debug($reqRes);
        }
    }
}
