<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Request;

use Magento\Payment\Gateway\Helper\SubjectReader;

/**
 * Class AddressDataBuilder
 */
class AddressDataBuilder extends GeneralDataBuilder
{
    public function build(array $buildSubject)
    {
        $paymentDO = SubjectReader::readPayment($buildSubject);
        $order = $paymentDO->getOrder();

        $billingAddress = $order->getBillingAddress();
        if (!empty($billingAddress->getRegionCode())) {
            $ssl_state = $billingAddress->getRegionCode();
        } else {
            $ssl_state = !empty($billingAddress->getRegion()) ? $this->customHelper->subStrFun($billingAddress->getRegion(),2) : '';
        }

        $result = [
            "ssl_email" => !empty($billingAddress->getEmail()) ? $billingAddress->getEmail() : '',
            "ssl_first_name" => $this->customHelper->subStrFun($billingAddress->getFirstname(), 20),
            "ssl_last_name" => $this->customHelper->subStrFun($billingAddress->getLastname(), 20),
            "ssl_company" => $this->customHelper->subStrFun($billingAddress->getCompany(), 50),
            "ssl_city" => $this->customHelper->subStrFun($billingAddress->getCity(), 20),
            "ssl_state" => $ssl_state,
            "ssl_country" => $billingAddress->getCountryId(),
            "ssl_phone" => $this->customHelper->subStrFun($billingAddress->getTelephone(), 20)
        ];

        if ($this->customHelper->getConfig('payment/risecommerce_elavon_option/zipcode_varification')) {
            $result['ssl_avs_address'] = $this->customHelper->subStrFun($this->getStreetAddress($billingAddress), 50);
            $result['ssl_avs_zip'] = $this->customHelper->subStrFun($billingAddress->getPostcode(), 9);
        }

        return $result;
    }

    public function getStreetAddress($address)
    {
        if (!empty($address->getStreetLine1())) {
            $strAdd = $address->getStreetLine1() .' '.$address->getStreetLine2();
        } else {
            $strAdd = implode(" ", $address->getStreet());
        }

        return $strAdd;
    }
}
