<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Request;

use Magento\Payment\Gateway\Helper\SubjectReader;

/**
 * Class LevelThreeDataBuilder
 */
class LevelThreeDataBuilder extends GeneralDataBuilder
{
    public function build(array $buildSubject)
    {
        $paymentDO = SubjectReader::readPayment($buildSubject);
        $payment = $paymentDO->getPayment();
        $order = $payment->getOrder();

        $result = [];
        if ($order->getShippingAddress()) {
            $ssl_shipping_amount = !empty($order->getBaseShippingAmount()) ? $order->getBaseShippingAmount() : '0.00';
            if ($this->customHelper->getPayInStoreCurrency()) {
                $ssl_shipping_amount = !empty($order->getShippingAmount()) ? $order->getShippingAmount() : '0.00';
            }

            $shippingaddress = $order->getShippingAddress();
            $ssl_ship_to_zip = $this->customHelper->subStrFun($shippingaddress->getPostcode(), 9);
            if (!empty($ssl_ship_to_zip) && strlen($ssl_ship_to_zip) >= 9) {
                $ssl_ship_to_zip = substr($ssl_ship_to_zip, 0, 9);
            }
            $ssl_ship_from_postal_code = $this->customHelper->getConfig(\Magento\Sales\Model\Order\Shipment::XML_PATH_STORE_ZIP);

            $ssl_level3_indicator = $this->customHelper->getConfig('payment/risecommerce_elavon_option/level_3') ? 'Y' : 'N';
            $result = [
                "ssl_level3_indicator" => $ssl_level3_indicator,
                "ssl_ship_to_zip" => $ssl_ship_to_zip,
                "ssl_ship_to_country" => $shippingaddress->getCountryId(),
                "ssl_shipping_amount" => number_format($ssl_shipping_amount, 2,'.', ''),
                "ssl_ship_from_postal_code" => $ssl_ship_from_postal_code
            ];
        }

        if (!empty($order->getBaseDiscountAmount())) {
            $ssl_discount_amount = $this->customHelper->getPayInStoreCurrency() ?  abs($order->getDiscountAmount()) : abs($order->getBaseDiscountAmount());
            $result['ssl_discount_amount'] = number_format($ssl_discount_amount, 2,'.', '');
        }

        return $result;
    }
}
