<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Request;

use Magento\Payment\Gateway\Helper\SubjectReader;

/**
 * Class LineItemDataBuilder
 */
class LineItemDataBuilder extends GeneralDataBuilder
{
    public function build(array $buildSubject)
    {
        $paymentDO = SubjectReader::readPayment($buildSubject);
        $payment = $paymentDO->getPayment();
        $order = $payment->getOrder();

        $result = [];
        $finalRowTotal = 0;
        $ssl_amount = number_format(SubjectReader::readAmount($buildSubject), 2,'.', '');
        $shipAmount = !empty($order->getBaseShippingAmount()) ? $order->getBaseShippingAmount() : '0.00';

        if ($this->customHelper->getConfig('payment/risecommerce_elavon_option/line_item')) {
            $itemCollection = [];
            foreach ($order->getItems() as $item) {
                if (!$item->getParentItem()) {
                    $taxable = !empty($item->getBaseTaxAmount()) ? 1 : 0;
                    $discountInd = !empty($item->getBaseDiscountAmount()) ? 1 : 0;
                    $rowTotal = $item->getBaseRowTotalInclTax() - $item->getBaseDiscountAmount();

                    $ssl_line_Item_unit_cost = $item->getBasePrice();
                    $ssl_line_item_discount_amount = $item->getBaseDiscountAmount();
                    $ssl_line_Item_tax_amount = $item->getBaseTaxAmount();
                    if ($this->customHelper->getPayInStoreCurrency()) {
                        $ssl_line_Item_unit_cost = $item->getPrice();
                        $ssl_line_item_discount_amount = $item->getDiscountAmount();
                        $ssl_line_Item_tax_amount = $item->getTaxAmount();
                        $rowTotal = $item->getRowTotalInclTax() - $item->getDiscountAmount();
                        $shipAmount = !empty($order->getShippingAmount()) ? $order->getShippingAmount() : '0.00';
                    }

                    $finalRowTotal = $finalRowTotal + $rowTotal;

                    $itemCollection['product'][] = [
                        "ssl_line_item_description" => trim(mb_substr($item->getName(), 0, 35)),
                        "ssl_line_Item_product_code" => $this->getCleanSku($item->getSku()),
                        "ssl_line_Item_quantity" => $item->getQtyOrdered(),
                        "ssl_line_Item_unit_cost" => $ssl_line_Item_unit_cost,

                        "ssl_line_Item_discount_indicator" => $discountInd,
                        "ssl_line_item_discount_amount" => $ssl_line_item_discount_amount,

                        "ssl_line_Item_tax_Indicator" => $taxable,
                        "ssl_line_Item_tax_rate" => $item->getTaxPercent(),
                        "ssl_line_Item_tax_amount" => $ssl_line_Item_tax_amount,
                        //"ssl_line_Item_tax_type" => 1,
                        "ssl_line_Item_total" => $rowTotal
                    ];
                }
            }
            $result['LineItemProducts'] = $itemCollection;
        }

        return $result;
    }

    function getCleanSku($sku) {
        $string = str_replace(' ', '-', $sku); // Replaces all spaces with hyphens.
        $string = preg_replace('/[^A-Za-z0-9\-]/', '', $sku); // Removes special chars.

        return  trim(mb_substr($string, 0, 12));
    }
}
