<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Request;

use Magento\Framework\Exception\LocalizedException;
use Magento\Payment\Model\MethodInterface;
use Risecommerce\Elavon\Observer\DataAssignObserver;
use Magento\Payment\Gateway\Helper\SubjectReader;

/**
 * Class TransactionDataBuilder
 */
class TransactionDataBuilder extends GeneralDataBuilder
{
    public function build(array $buildSubject)
    {
        $paymentDO = SubjectReader::readPayment($buildSubject);
        $payment = $paymentDO->getPayment();

        if ($this->customHelper->getEnableCaptcha() != 0 &&
            $this->customHelper->checkAdmin() != 'adminhtml'
        ) {
            $CAPTCHA = $payment->getAdditionalInformation(DataAssignObserver::G_CAPTCHA);
            $verifyC = $this->customHelper->verifyGC($CAPTCHA);
            if ($verifyC != '') {
                throw new LocalizedException(__($verifyC));
            }
        }

        $dt = \DateTime::createFromFormat('Y', $payment->getData(DataAssignObserver::CC_EXP_YEAR));
        $expYear = trim($payment->getData(DataAssignObserver::CC_EXP_YEAR));
        if ($dt) {
            $expYear = $dt->format('y');
        }
        $expMonth = sprintf("%02d", $payment->getData(DataAssignObserver::CC_EXP_MONTH));
        $expDate = $expMonth.$expYear;
        $cvv = $payment->getData(DataAssignObserver::CC_CID);
        $ssl_transaction_type = $this->customHelper->getConfig('payment/risecommerce_elavon_option/payment_action') == MethodInterface::ACTION_AUTHORIZE ? "ccauthonly" : "ccsale";

        $result = [
            'ssl_transaction_type' => $ssl_transaction_type,
            'ssl_card_number' => $payment->getData(DataAssignObserver::CC_NUMBER),
            'ssl_exp_date' => $expDate,
            'ssl_cvv2cvc2_indicator' => '1',
            'ssl_cvv2cvc2' => $cvv
        ];

        return $result;
    }
}
