<?php
/**
 * 
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Response;

use Magento\Payment\Gateway\Helper\SubjectReader;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Sales\Model\Order\Payment;
use Risecommerce\Elavon\Helper\Data as CustomHelper;
use Risecommerce\Elavon\Observer\DataAssignObserver;

/**
 * Class PaymentDetailsHandler
 */
class PaymentDetailsHandler implements HandlerInterface
{
    /**
     * @var CustomHelper
     */
    protected $customHelper;

    /**
     * @param CustomHelper $customHelper
     */
    public function __construct(
        CustomHelper $customHelper
    ) {
        $this->customHelper = $customHelper;
    }

    /**
     * @inheritdoc
     */
    public function handle(array $handlingSubject, array $response)
    {
        $paymentDO = SubjectReader::readPayment($handlingSubject);
        $payment = $paymentDO->getPayment();
        $transId = $response['ssl_txn_id'];
        $payment->setTransactionId($transId);
        $payment->setCcTransId($transId);
        $payment->setLastTransId($transId);
        $payment->setCcStatus($response['ssl_result']);
        $payment->setAdditionalInformation('order_tra_id', $transId);

        if (!empty($response['ssl_approval_code'])) {
            $payment->setAdditionalInformation('ssl_approval_code', $response['ssl_approval_code']);
        }
        if (!empty($response['ssl_card_number'])) {
            $payment->setCcLast4(substr($response['ssl_card_number'], -4));
            $payment->setAdditionalInformation('cc_num', 'xxxx-' . substr($response['ssl_card_number'], -4));
        }
        if (isset($response['ssl_card_short_description'])) {
            $payment->setCcType($response['ssl_card_short_description']);
            $payment->setAdditionalInformation('cc_type', $response['ssl_card_short_description']);
        }

        $payment->setIsTransactionClosed($this->shouldCloseTransaction());
        $closed = $this->shouldCloseParentTransaction($payment);
        $payment->setShouldCloseParentTransaction($closed);
    }

    /**
     * Whether transaction should be closed
     *
     * @return bool
     */
    protected function shouldCloseTransaction(): bool
    {
        return false;
    }

    /**
     * Whether parent transaction should be closed
     *
     * @param Payment $orderPayment
     * @return bool
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    protected function shouldCloseParentTransaction(Payment $payment): bool
    {
        return false;
    }
}
