<?php
namespace Risecommerce\Elavon\Gateway\Response;

use Magento\Payment\Gateway\Helper\SubjectReader;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Payment\Model\InfoInterface;
use Magento\Sales\Api\Data\OrderPaymentExtensionInterface;
use Magento\Sales\Api\Data\OrderPaymentExtensionInterfaceFactory;
use Magento\Vault\Api\Data\PaymentTokenFactoryInterface;
use Magento\Vault\Api\Data\PaymentTokenInterface;
use Risecommerce\Elavon\Observer\DataAssignObserver;

/**
 * Class VaultDetailsHandler
 */
class VaultDetailsHandler implements HandlerInterface
{
    /**
     * @var \Risecommerce\Elavon\Helper\Data
     */
    public $customHelper;

    /**
     * @var PaymentTokenFactoryInterface
     */
    protected $paymentTokenFactory;

    /**
     * @var OrderPaymentExtensionInterfaceFactory
     */
    protected $paymentExtensionFactory;

    /**
     * @var Json
     */
    private $serializer;

    public function __construct(
        \Risecommerce\Elavon\Helper\Data $helper,
        PaymentTokenFactoryInterface $paymentTokenFactory,
        OrderPaymentExtensionInterfaceFactory $paymentExtensionFactory,
        Json $serializer = null
    ) {
        $this->customHelper = $helper;
        $this->paymentTokenFactory = $paymentTokenFactory;
        $this->paymentExtensionFactory = $paymentExtensionFactory;
        $this->serializer = $serializer ?: ObjectManager::getInstance()->get(Json::class);
    }

    /**
     * @inheritdoc
     */
    public function handle(array $handlingSubject, array $response)
    {
        $paymentDO = SubjectReader::readPayment($handlingSubject);
        $payment = $paymentDO->getPayment();

        $saveCard = $payment->getAdditionalInformation(DataAssignObserver::SAVE_CARD);
        if ($saveCard == '1') {
            $paymentToken = $this->getVaultPaymentToken($response, $payment);
            if (null !== $paymentToken) {
                $extensionAttributes = $this->getExtensionAttributes($payment);
                $extensionAttributes->setVaultPaymentToken($paymentToken);
            }
        }
    }

    protected function getVaultPaymentToken($response, $payment)
    {
        if (empty($response['ssl_token'])) {
            return null;
        }
        $token = $response['ssl_token'];

        /** @var PaymentTokenInterface $paymentToken */
        $paymentToken = $this->paymentTokenFactory->create(PaymentTokenFactoryInterface::TOKEN_TYPE_CREDIT_CARD)->load($token, 'gateway_token');
        if (!$paymentToken->getEntityId()) {
            $paymentToken = $this->paymentTokenFactory->create(PaymentTokenFactoryInterface::TOKEN_TYPE_CREDIT_CARD);
        } else {
            $paymentToken->setIsActive(1);
            $paymentToken->setIsVisible(1);
        }

        //$paymentToken = $this->paymentTokenFactory->create(PaymentTokenFactoryInterface::TOKEN_TYPE_CREDIT_CARD);
        $paymentToken->setGatewayToken($token);
        $paymentToken->setExpiresAt($this->getExpirationDate($payment));

        $ccType = '';
        if (!empty($response['ssl_card_short_description'])) {
            $ccType = $this->customHelper->getCcTypeCodeByName($response['ssl_card_short_description']);
        }
        $ccLast4 = '';
        if (!empty($response['ssl_card_number'])) {
            $ccLast4 = substr($response['ssl_card_number'], -4);
        }
        $expMon = sprintf("%02d", $payment->getData(DataAssignObserver::CC_EXP_MONTH));
        $expYr = $payment->getData(DataAssignObserver::CC_EXP_YEAR);
        $expirationDate = $expMon.'/'.$expYr;

        $paymentToken->setTokenDetails($this->convertDetailsToJSON([
            'type' => $ccType,
            'maskedCC' => $ccLast4,
            'expirationDate' => $expirationDate
        ]));

        if ($paymentToken->getEntityId()) {
            try {
                $paymentToken->save();
            } catch (\Exception $e) {
                return false;
            }
        }

        return $paymentToken;
    }

    private function convertDetailsToJSON($details)
    {
        $json = $this->serializer->serialize($details);
        return $json ? $json : '{}';
    }

    private function getExtensionAttributes(InfoInterface $payment)
    {
        $extensionAttributes = $payment->getExtensionAttributes();
        if (null === $extensionAttributes) {
            $extensionAttributes = $this->paymentExtensionFactory->create();
            $payment->setExtensionAttributes($extensionAttributes);
        }
        return $extensionAttributes;
    }

    private function getExpirationDate($payment): string
    {
        $expMon = sprintf("%02d", $payment->getData(DataAssignObserver::CC_EXP_MONTH));
        $expYr = $payment->getData(DataAssignObserver::CC_EXP_YEAR);

        $expDate = new \DateTime(
            $expYr
            . '-'
            . $expMon
            . '-'
            . '01'
            . ' '
            . '00:00:00',
            new \DateTimeZone('UTC')
        );
        $expDate->add(new \DateInterval('P1M'));

        return $expDate->format('Y-m-d 00:00:00');
    }
}
