<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Gateway\Validator;

use Magento\Payment\Gateway\Helper\SubjectReader;
use Magento\Payment\Gateway\Validator\AbstractValidator;

/**
 * Class GeneralResponseValidator
 */
class GeneralResponseValidator extends AbstractValidator
{
    public function validate(array $validationSubject)
    {
        $response = SubjectReader::readResponse($validationSubject);
        $errorMessages = [];
        $errorCode = [];
        $validationResult = $this->responseCode($response) && $this->validateTxnId($response);

        if (!$validationResult) {
            if (!empty($response['errorMessage']) & !empty($response['errorCode'])) {
                $errorCode[] = $response['errorCode'];
                $errorMessages[] = __($response['errorMessage']);
            }
        }

        return $this->createResult($validationResult, $errorMessages, $errorCode);
    }

    /**
     * @param $response
     * @return bool
     */
    protected function responseCode($response)
    {
        return isset($response['ssl_result']) && $response['ssl_result'] == '0';
    }

    /**
     * @param $response
     * @return bool
     */
    protected function validateTxnId($response)
    {
        return !empty($response['ssl_txn_id']);
    }
}
