<?php
/**
 * Elavon Module Helper.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */

namespace Risecommerce\Elavon\Helper;

use Magento\Framework\App\State;
use Magento\Payment\Model\Config as PaymentConfig;
use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Backend\Helper\Data as BackendHelper;
use Risecommerce\Elavon\Model\Method\Logger as RcLogger;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\Encryption\EncryptorInterface
     */
    protected $_encryptor;

    /**
     * @var \Magento\Config\Model\ResourceModel\Config
     */
    protected $resourceConfig;

    /**
     * @var \Magento\Directory\Model\RegionFactory
     */
    protected $_regionFactory;

    /**
     * @var \Magento\Directory\Model\CountryFactory
     */
    protected $_countryFactory;

    /**
     * @var \Magento\Directory\Model\CurrencyFactory
     */
    protected $currencyFactory;

    /**
     * @var PaymentConfig
     */
    protected $paymentConfig;

    /** *@var State */
    protected $state;

    /** *@var PriceCurrencyInterface */
    protected $priceCurrencyInterface;

    /** @var BackendHelper */
    protected $backendHelper;

    /** @var RcLogger */
    private $rcLogger;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Encryption\EncryptorInterface $encryptor
     * @param \Magento\Config\Model\ResourceModel\Config $resourceConfig
     * @param \Magento\Directory\Model\RegionFactory $regionFactory
     * @param \Magento\Directory\Model\CountryFactory $countryFactory
     * @param \Magento\Directory\Model\CurrencyFactory $currencyFactory
     * @param PaymentConfig $paymentConfig
     * @param State $state
     * @param PriceCurrencyInterface $priceCurrencyInterface
     * @param BackendHelper $backendHelper
     * @param RcLogger $rcLogger
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
        \Magento\Config\Model\ResourceModel\Config $resourceConfig,
        \Magento\Directory\Model\RegionFactory $regionFactory,
        \Magento\Directory\Model\CountryFactory $countryFactory,
        \Magento\Directory\Model\CurrencyFactory $currencyFactory,
        PaymentConfig $paymentConfig,
        State $state,
        PriceCurrencyInterface $priceCurrencyInterface,
        BackendHelper $backendHelper,
        RcLogger $rcLogger
    ) {
        $this->_storeManager = $storeManager;
        $this->_encryptor = $encryptor;
        $this->resourceConfig = $resourceConfig;
        $this->_regionFactory = $regionFactory;
        $this->_countryFactory = $countryFactory;
        $this->currencyFactory = $currencyFactory;
        $this->paymentConfig = $paymentConfig;
        $this->priceCurrencyInterface = $priceCurrencyInterface;
        $this->backendHelper = $backendHelper;
        $this->rcLogger = $rcLogger;
        parent::__construct($context);
        $this->state = $state;
    }

    /**
     * Get Configuration value from admin.
     */
    public function getConfig($config_path, $storeId = null)
    {
        return $this->scopeConfig->getValue(
            $config_path,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Get value of Merchang ID.
     */
    public function getMerchantId($storeId = null)
    {
        return $this->getConfig('payment/risecommerce_elavon_general/merchant_id', $storeId);
    }

    /**
     * Get value of User ID.
     */
    public function getUserId($storeId = null)
    {
        $user_id = $this->getConfig('payment/risecommerce_elavon_general/user_id', $storeId);
        return $this->_encryptor->decrypt($user_id);
    }

    /**
     * Get value of Elavon Pin.
     */
    public function getElavonPin($storeId = null)
    {
        $elavon_pin = $this->getConfig('payment/risecommerce_elavon_general/elavon_pin', $storeId);
        return $this->_encryptor->decrypt($elavon_pin);
    }

    /**
     * Get value of Payment Mode.
     */
    public function getPaymentMode($storeId = null)
    {
        $payment_mode = $this->getConfig('payment/risecommerce_elavon_general/payment_mode', $storeId);
        return $payment_mode == 1 ? true : false;
    }

    /**
     * Get Hosted Form Action URL
     *
     * @return string
     */
    public function getFormAction()
    {
        return $this->getPaymentMode() == true ?
            'https://api.demo.convergepay.com/hosted-payments' :
            'https://api.convergepay.com/hosted-payments';
    }

    /**
     * Get Hosted Form Token URL
     *
     * @return string
     */
    public function getFormToken()
    {
        return $this->getPaymentMode() == true ?
            'https://api.demo.convergepay.com/hosted-payments/transaction_token' :
            'https://api.convergepay.com/hosted-payments/transaction_token';
    }

    /**
     * @return string
     */
    public function getPayInStoreCurrency()
    {
        return $this->getConfig('payment/risecommerce_elavon_general/payinstorecurrency');
    }

    /**
     * Get value of Debug Setting.
     */
    public function isDebugEnabled()
    {
        return $this->getConfig('payment/risecommerce_elavon_general/debug');
    }

    /**
     * Get value of licence key from admin
     */
    public function licencekey()
    {
        return $this->getConfig('risecommerce_elavon/general/licencekey');
    }

    /**
     * return area code.
     */
    public function checkAdmin()
    {
        return $this->state->getAreaCode();
    }

    /**
     * Get value of secure URL from admin
     */
    public function surl()
    {
        return "aHR0cHM6dHdheXMuY29tL20ydmVyaWZ5bGljLnBocA==";
    }

    /**
     * Get value of licence key from admin
     */
    public function act()
    {
        $dt_db_blank = $this->getConfig('risecommerce_elavon/general/lcstatus');
        if ($dt_db_blank == '') {
            $isMultiStore =  $this->getConfig('risecommerce_elavon/general/ismultistore');
            $u = $this->_storeManager->getStore()->getBaseUrl();
            if ($isMultiStore == 1) {
                $u = $this->backendHelper->getHomePageUrl();
            }
            $l = $this->getConfig('risecommerce_elavon/general/licencekey');
            $surl = base64_decode($this->surl());
            $url = $surl."?u=".$u."&l=".$l."&extname=m2_elavon";
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_URL, $url);
            $result=true;
            curl_close($ch);
            if (!$result) {
                return "SXNzdWUgd2l0aCB5b3VyIFJvb3R3YXlzIGV4dGVuc2lvbiBsaWNlbnNlIGtleSwgcGxlYXNlIGNvbnRhY3QgPGEgaHJlZj0ibWFpbHRvOmhlbHBAcm9vdHdheXMuY29tIj5oZWxwQHJvb3R3YXlzLmNvbTwvYT4=";
            }
//            $act_data = json_decode($result, true);
  //          if (isset($act_data['status']) && $act_data['status'] == '0') {
//                return "SXNzdWUgd2l0aCB5b3VyIFJvb3R3YXlzIGV4dGVuc2lvbiBsaWNlbnNlIGtleSwgcGxlYXNlIGNvbnRhY3QgPGEgaHJlZj0ibWFpbHRvOmhlbHBAcm9vdHdheXMuY29tIj5oZWxwQHJvb3R3YXlzLmNvbTwvYT4=";
  //          } else {
//                $this->resourceConfig->saveConfig('risecommerce_elavon/general/lcstatus', $l, 'default', 0);
  //          }
        }

        return '';
    }

    private function isJson($string){
        return is_string($string) && is_array(json_decode($string, true)) ? true : false;
    }

    /**
     * Get value of Post URL.
     *
     * @return string
     */
    public function getPostUrl()
    {
        return $this->getPaymentMode() == true ?
            $this->getConfig('payment/risecommerce_elavon_general/sandbox_gateway_url') :
            $this->getConfig('payment/risecommerce_elavon_general/gateway_url');
    }

    /**
     * Get value of Region Code.
     */
    public function getRegionCode($shipperRegionId)
    {
        $shipperRegion = $this->_regionFactory->create()->load($shipperRegionId);
        return $shipperRegion->getCode();
    }

    /**
     * Get value of Country ID.
     */
    public function getCountryName($countryCode)
    {
        $country = $this->_countryFactory->create()->loadByCode($countryCode);
        return $country->getName();
    }

    /**
     * @return string
     */
    public function getStoreBaseAmount($amountValue = 0, $currencyCodeFrom = null, $currencyCodeTo = null)
    {
        if ($currencyCodeFrom == $currencyCodeTo) {
            return $amountValue;
        }

        /** @var float $rate */
        // Get rate
        $rate = $this->currencyFactory->create()->load($currencyCodeFrom)->getAnyRate($currencyCodeTo);
        // Get amount in new currency
        $amountValue = $amountValue * $rate;

        return $this->priceCurrencyInterface->round($amountValue);
    }

    /**
     * Retrieve credit card name by code.
     */
    public function getCcTypeNameByCode($code)
    {
        $ccTypes = $this->paymentConfig->getCcTypes();
        if (isset($ccTypes[$code])) {
            return $ccTypes[$code];
        } else {
            return false;
        }
    }

    /**
     * Retrieve credit card code by name.
     */
    public function getCcTypeCodeByName($name)
    {
        $ccTypes = $this->paymentConfig->getCcTypes();
        $key = array_search(strtolower($name), array_map('strtolower', $ccTypes));
        if ($key != '') {
            return $key;
        } else {
            return false;
        }
    }

    /**
     * Retrieve credit card types.
     */
    public function getCcTypes()
    {
        $ccTypes = $this->paymentConfig->getCcTypes();
        if (is_array($ccTypes)) {
            return $ccTypes;
        } else {
            return false;
        }
    }

    /**
     * Retrieve selected credit card from config.
     */
    public function getCcAvailableCardTypes($country = null)
    {
        $cType = $this->getConfig('payment/risecommerce_elavon_option/cctypes');
        if (empty($cType)) {
            return [];
        }
        $types = array_flip(explode(',', $cType));
        $mergedArray = [];

        if (is_array($types)) {
            foreach (array_keys($types) as $type) {
                $types[$type] = $this->getCcTypeNameByCode($type);
            }
        }

        //preserve the same credit card order
        $allTypes = $this->getCcTypes();
        if (is_array($allTypes)) {
            foreach ($allTypes as $ccTypeCode => $ccTypeName) {
                if (array_key_exists($ccTypeCode, $types)) {
                    $mergedArray[$ccTypeCode] = $ccTypeName;
                }
            }
        }

        return $mergedArray;
    }

    /**
     * @return boolean
     */
    public function getEnableCaptcha()
    {
        return $this->getConfig('payment/risecommerce_elavon_option/enable_captcha');
    }

    /**
     * @return string
     */
    public function getCaptchaSiteKey()
    {
        if ($this->getEnableCaptcha() == 3) {
            $val = $this->getConfig('payment/risecommerce_elavon_option/v3_site_key');
        } else {
            $val = $this->getConfig('payment/risecommerce_elavon_option/site_key');
        }
        return $this->_encryptor->decrypt($val);
    }

    /**
     * @return string
     */
    public function getCaptchaSecretKey()
    {
        if ($this->getEnableCaptcha() == 3) {
            $val = $this->getConfig('payment/risecommerce_elavon_option/v3_secret_key');
        } else {
            $val = $this->getConfig('payment/risecommerce_elavon_option/secret_key');
        }
        return $this->_encryptor->decrypt($val);
    }

    public function verifyGC($gCaptcha)
    {
        $isWrongCaptcha = 1;
        if (!empty($gCaptcha)) {
            $post_data = http_build_query(
                [
                    'secret' => $this->getCaptchaSecretKey(),
                    'response' => $gCaptcha,
                    'remoteip' => $this->getClientIp()
                ]
            );

            $opts = ['http' =>
                [
                    'method'  => 'POST',
                    'header'  => 'Content-type: application/x-www-form-urlencoded',
                    'content' => $post_data
                ]
            ];

            $context  = stream_context_create($opts);
            $response = file_get_contents('https://www.google.com/recaptcha/api/siteverify', false, $context);
            $result = json_decode($response);

            if (isset($result->success)) {
                if ($this->getEnableCaptcha() == 3) {
                    if (isset($result->score)) {
                        $captchaMinScore = 0.5;
                        $captchaScore = $this->getConfig('payment/risecommerce_elavon_option/c_minimum_score');
                        if ($captchaScore != '') {
                            $captchaMinScore = $captchaScore;
                        }
                        if ($result->success == 1 && (float)$result->score >= (float)$captchaMinScore) {
                            $isWrongCaptcha = 0;
                        }
                    }
                } else {
                    if ($result->success == 1) {
                        $isWrongCaptcha = 0;
                    }
                }
            }
        }

        $captchaErrorMsg = '';
        if ($isWrongCaptcha == 1) {
            $captchaErrorMsg = __('reCAPTCHA verification failed.');
            $v2Msg = $this->getConfig('payment/risecommerce_elavon_option/v2_validation_failure_message');
            $v3Msg = $this->getConfig('payment/risecommerce_elavon_option/v3_validation_failure_message');
            if ($this->getEnableCaptcha() == 3) {
                $captchaErrorMsg = __('You cannot proceed with such operation, your reCAPTCHA reputation is too low.');
                if ($v3Msg != '') {
                    $captchaErrorMsg = $v3Msg;
                }
            } else {
                $captchaErrorMsg = __('reCAPTCHA verification failed.');
                if ($v2Msg != '') {
                    $captchaErrorMsg = $v2Msg;
                }
            }
        }

        return $captchaErrorMsg;
    }

    public function getClientIp()
    {
        if (getenv('HTTP_CLIENT_IP')) {
            $ipaddress = getenv('HTTP_CLIENT_IP');
        } elseif (getenv('HTTP_X_FORWARDED_FOR')) {
            $ips = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
            $ipaddress = trim($ips[0]); //real IP address behind proxy IP
        } elseif (getenv('HTTP_X_FORWARDED')) {
            $ipaddress = getenv('HTTP_X_FORWARDED');
        } elseif (getenv('HTTP_FORWARDED_FOR')) {
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        } elseif (getenv('HTTP_FORWARDED')) {
            $ipaddress = getenv('HTTP_FORWARDED');
        } elseif (getenv('REMOTE_ADDR')) {
            $ipaddress = getenv('REMOTE_ADDR');
        } else {
            $ipaddress = 'UNKNOWN';
        }

        if (strlen($ipaddress) >= 40) {
            $ipaddress = substr($ipaddress, 0, 40);
        }

        return $ipaddress;
    }

    public function getStoreId()
    {
        return $this->_storeManager->getStore()->getId();
    }

    /**
     *  Request field validation
     */
    public function subStrFun($val, $len)
    {
        if (!empty($val) && strlen($val) >= $len) {
            $val = substr($val, 0, $len);
        }

        return $val;
    }

    public function rcLogger($req, $res)
    {
        $reqRes = ["Request" => $req, "Response" => $res];
        $this->rcLogger->debug($reqRes);
    }
}
