<?php
/**
 * Elavon Payment Model.
 *
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Model;

use Magento\Framework\Exception\LocalizedException;
use Magento\Payment\Model\Method\AbstractMethod;

class PaymentHosted extends AbstractMethod
{
    const CODE_HOSTED = "risecommerce_elavonhosted_option";
    protected $_code = self::CODE_HOSTED;
    protected $_formBlockType = 'Risecommerce\Elavon\Block\Form';
    protected $_infoBlockType = 'Risecommerce\Elavon\Block\InfoHostedPage';

    protected $_isGateway = true;
    protected $_canOrder = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_canCapturePartial = true;
    protected $_canRefund = true;
    protected $_canRefundInvoicePartial = true;
    protected $_canVoid = true;
    protected $_canUseInternal = false;
    protected $_canUseCheckout = true;
    protected $_canFetchTransactionInfo = true;

    /**
     * @var \Risecommerce\Elavon\Helper\Data
     */
    protected $customHelper;

    /**
     * PaymentHosted constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory
     * @param \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory
     * @param \Magento\Payment\Helper\Data $paymentData
     * @param \Risecommerce\Elavon\Helper\Data $customHelper
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Payment\Model\Method\Logger $logger
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Risecommerce\Elavon\Helper\Data $customHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );
        $this->customHelper = $customHelper;
    }

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if ($this->customHelper->checkAdmin() == 'adminhtml') {
            return false;
        }

        return parent::isAvailable($quote);
    }

    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $posts = $payment->getData('additional_information');
        if (empty($posts['order_tra_id'])) {
            throw new LocalizedException(__('Error Processing the request. No response data found from the payment server'));
        } else {
            $traId = $posts['order_tra_id'];
            $payment->setTransactionId($traId);
            $payment->setLastTransId($traId);
            $payment->setIsTransactionClosed(0);
        }

        return $this;
    }

    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        if ($payment->getTransactionId() != '') {
            if ($amount<=0) {
                throw new LocalizedException(__('Error: Invalid amount for the capture.'));
            }
            $ref_id = $payment->getTransactionId();
            $order = $payment->getOrder();
            if ($this->customHelper->getPayInStoreCurrency()) {
                $amount = $this->customHelper->getStoreBaseAmount($amount, $order->getBaseCurrencyCode(), $order->getOrderCurrencyCode());
            }
            $trn_type = 'c';
            $result = $this->createCharge($payment, $amount, $trn_type, $ref_id);
            if (!isset($result->ssl_result)) {
                $errorMessage = $this->errorMsg($result);
                throw new LocalizedException(__($errorMessage));
            }
            if ($result->ssl_result != '0') {
                $errorMessage = $this->errorMsg($result);
                throw new LocalizedException(__($errorMessage));
            } else {
                if ($result->ssl_result == '0' && $result->ssl_txn_id != '') {
                    $payment->setTransactionId($result->ssl_txn_id);
                    $payment->setIsTransactionClosed(1);
                    $payment->setAdditionalInformation('order_tra_id', (string)$result->ssl_txn_id);
                    $payment->setAdditionalInformation('payment_status', 'Captured');
                    if ($ref_id != 0) {
                        $payment->setAdditionalInformation('last_transaction_id', (string)$result->ssl_txn_id);
                    } else {
                        $payment->setAdditionalInformation('real_transaction_id', (string)$result->ssl_txn_id);
                        if (isset($result->ssl_avs_response)) {
                            $payment->setAdditionalInformation('avs_result', (string)$result->ssl_avs_response);
                        }
                        if (isset($result->ssl_cvv2_response)) {
                            $payment->setAdditionalInformation('cvd_result', (string)$result->ssl_cvv2_response);
                        }
                    }
                } else {
                    throw new LocalizedException(__('There is an error in payment processing, please try again.'));
                }
            }
        } else {
            $posts = $payment->getData('additional_information');
            if (empty($posts['order_tra_id'])) {
                throw new LocalizedException(__('Error Processing the request. No response data found from the payment server'));
            } else {
                $traId = $posts['order_tra_id'];
                $payment->setTransactionId($traId);
                $payment->setLastTransId($traId);
                $payment->setIsTransactionClosed(1);
            }
        }

        return $this;
    }

    public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        if ($payment->getRefundTransactionId() && $amount > 0) {
            $trn_type = 'r';
            if ($this->customHelper->getPayInStoreCurrency()) {
                $creditmemo = $payment->getCreditmemo();
                $amount = $creditmemo->getGrandTotal();
            }
            $result = $this->createCharge($payment, $amount, $trn_type, $payment->getRefundTransactionId());
            if ($result->ssl_result == '0') {
                $payment->setAdditionalInformation('last_transaction_id', (string)$result->ssl_txn_id);
                $payment->setAdditionalInformation('payment_status', 'Refunded');
            } else {
                throw new LocalizedException(__('There is an error in refund processing. '.$result->errorMessage));
            }
        } else {
            throw new LocalizedException(__('There is an error in payment processing. Please contact Elavon support.'));
        }
        return $this;
    }

    /**
     * @param \Magento\Payment\Model\InfoInterface $payment
     *
     */
    public function void(\Magento\Payment\Model\InfoInterface $payment)
    {
        throw new LocalizedException(__('VOID is not possible with Elavon/Converge Payment Gateways and Magento 2.
        Instead, you can cancel an order from Magento by clicking the "Cancel" button.
        NOTE: This cancel is offline and it will not cancel/void the order in Elavon,
        it will cancel order from the Magento only.'));
    }

    private function createCharge($payment, $amount, $trn_type, $ref_id)
    {
        $order = $payment->getOrder();
        $orderStoreId = $order->getStoreId();
        $ssl_merchant_id = $this->customHelper->getMerchantId($orderStoreId);
        $ssl_user_id = $this->customHelper->getUserId($orderStoreId);
        $ssl_pin = $this->customHelper->getElavonPin($orderStoreId);

        if ($trn_type == 'a') {
            $ssl_transaction_type = 'ccauthonly';
        } elseif ($trn_type == 'c' && $ref_id != '0') {
            $ssl_transaction_type = 'cccomplete';
        } elseif ($trn_type == 'r') {
            $ssl_transaction_type = 'ccreturn';
        } elseif ($trn_type == 'v') {
            $ssl_transaction_type = 'ccvoid';
        } else {
            $ssl_transaction_type = 'ccsale';
        }

        if ($ssl_transaction_type == 'cccomplete' || $ssl_transaction_type == 'ccreturn') {
            $ssl_amount = number_format($amount, 2, '.', '');
            $ssl_description = sprintf('Order # %s', $order->getIncrementId());

            $charge_id = str_replace('-capture', '', $ref_id);
            $xml = "<txn>";
            $xml .= "<ssl_merchant_id>" . $ssl_merchant_id . "</ssl_merchant_id>
                    <ssl_pin>" . $ssl_pin . "</ssl_pin>
                    <ssl_user_id>" . $ssl_user_id . "</ssl_user_id>
                    <ssl_transaction_type>".$ssl_transaction_type."</ssl_transaction_type>
                    <ssl_description>".$ssl_description."</ssl_description>
                    <ssl_txn_id>".$charge_id."</ssl_txn_id>
                    <ssl_amount>" . $ssl_amount . "</ssl_amount>";
            if ($trn_type== 'c' && $amount < $order->getGrandTotal()) {
                $xml .= "<ssl_partial_shipment_flag>Y</ssl_partial_shipment_flag>";
            }
            $xml .= '</txn>';
        }

        if ($ssl_transaction_type == 'ccvoid') {
            $void_id = str_replace('-void', '', $ref_id);
            $xml = "<txn><ssl_merchant_id>" . $ssl_merchant_id . "</ssl_merchant_id>
                    <ssl_pin>" . $ssl_pin . "</ssl_pin>
                    <ssl_user_id>" . $ssl_user_id . "</ssl_user_id>
                    <ssl_transaction_type>".$ssl_transaction_type."</ssl_transaction_type>
                    <ssl_txn_id>".$void_id."</ssl_txn_id>
                </txn>";
        }

        $postURL = $this->customHelper->getPostUrl();
        $replaceFrom = ["%", "=", "&"];
        $replaceTo = ["percentage", "equal", "and"];
        $xmlNew = str_replace($replaceFrom, $replaceTo, $xml);
        $postData = "xmldata=" . URLEncode($xmlNew);
        $session = curl_init();
        $header[] = "Content-Length: " . strlen($postData);
        $header[] = "Content-Type: application/x-www-form-urlencoded";
        curl_setopt($session, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($session, CURLOPT_URL, $postURL);
        curl_setopt($session, CURLOPT_POST, true);
        curl_setopt($session, CURLOPT_POSTFIELDS, $postData);
        curl_setopt($session, CURLOPT_HEADER, false);
        curl_setopt($session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($session);
        $result = simplexml_load_string($response);

        if ($this->customHelper->isDebugEnabled()) {
            $reqOut = $this->delete_all_between('<ssl_merchant_id>', '</ssl_exp_date>', $xml);
            $resOut = $this->delete_all_between('<ssl_txn_id>', '</ssl_account_balance>', $response);
            $this->customHelper->rcLogger($reqOut, $resOut);
        }
        return $result;
    }

    public function getClientIp()
    {
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP')) {
            $ipaddress = getenv('HTTP_CLIENT_IP');
        } elseif (getenv('HTTP_X_FORWARDED_FOR')) {
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        } elseif (getenv('HTTP_X_FORWARDED')) {
            $ipaddress = getenv('HTTP_X_FORWARDED');
        } elseif (getenv('HTTP_FORWARDED_FOR')) {
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        } elseif (getenv('HTTP_FORWARDED')) {
            $ipaddress = getenv('HTTP_FORWARDED');
        } elseif (getenv('REMOTE_ADDR')) {
            $ipaddress = getenv('REMOTE_ADDR');
        } else {
            $ipaddress = 'UNKNOWN';
        }
        return $ipaddress;
    }

    function delete_all_between($beginning, $end, $string)
    {
        $beginningPos = strpos($string, $beginning);
        $endPos = strpos($string, $end);
        if ($beginningPos === false || $endPos === false) {
            return $string;
        }

        $textToDelete = substr($string, $beginningPos, ($endPos + strlen($end)) - $beginningPos);

        return $this->delete_all_between($beginning, $end, str_replace($textToDelete, '', $string)); // recursion to ensure all occurrences are replaced
    }

    public function errorMsg($result)
    {
        $errorMessage = 'There has been an error processing your request.';
        if (isset($result->ssl_result_message)) {
            $errorMessage .= ' Error from the payment server: '.$result->ssl_result_message;
        } else {
            if (isset($result->errorMessage)) {
                $errorMessage .= ' Error from the payment server: '.$result->errorMessage;
            }
        }
        return $errorMessage;
    }
}
