<?php
namespace Risecommerce\Elavon\Model\Request;

use Magento\Framework\HTTP\Client\Curl;
use Risecommerce\Elavon\Helper\Data;
use Risecommerce\Elavon\Model\Method\Logger as RcLogger;

class Api
{
    /**
     * Elavon helper.
     *
     * @var Data
     */
    protected $customHelper;

    /**
     * @var Curl
     */
    protected $curl;

    /**
     * @var RcLogger
     */
    private $rcLogger;

    /**
     * @param Data $customHelper
     * @param Curl $curl
     * @param RcLogger $rcLogger
     * @param array $data
     */
    public function __construct(
        Data $customHelper,
        Curl $curl,
        RcLogger $rcLogger,
        array $data = []
    ) {
        $this->customHelper = $customHelper;
        $this->curl = $curl;
        $this->rcLogger = $rcLogger;
    }

    /**
     * Common function for all operations.
     */
    protected function _sendRestRequest($postData, $curlMethod = 'POST')
    {
        $url = $this->customHelper->getPostUrl($this->customHelper->getStoreId());
        $headers = ["Content-Type" => "application/x-www-form-urlencoded", "Content-length" => strlen($postData)];

        if ($curlMethod == 'POST') {
            $this->curl->setHeaders($headers);
            $this->curl->post($url, $postData);
        } else {
            $this->curl->get($url);
        }
        $response = $this->curl->getBody();
        $new = simplexml_load_string($response);
        $con = json_encode($new);
        $resultNew = json_decode($con, true);

        return $resultNew;
    }

    /**
     * Get status of the order.
     *
     * @return array
     */
    public function deleteProfile($token)
    {
        $request = [
            "ssl_merchant_id" =>  $this->customHelper->getMerchantId($this->customHelper->getStoreId()),
            "ssl_pin" => $this->customHelper->getElavonPin($this->customHelper->getStoreId()),
            "ssl_user_id" => $this->customHelper->getUserId($this->customHelper->getStoreId()),
            "ssl_transaction_type" => "ccdeletetoken",
            "ssl_token" => $token
        ];

        $xml = $this->convertToXml($request);
        $xml = !empty($xml) ? trim(str_replace('<?xml version="1.0"?>', '', $xml)) : $xml;
        $replaceFrom = ["%", "=", "&"];
        $replaceTo = ["percentage", "equal", "and"];
        $xmlNew = str_replace($replaceFrom, $replaceTo, $xml);
        $postData = "xmldata=" . URLEncode($xmlNew);
        $restResponse = $this->_sendRestRequest($postData, "POST");

        $reqRes = ["Delete Vault Token Request" => $request, "Delete Vault Token Response" => $restResponse];
        $this->rcLogger->debug($reqRes);

        return $restResponse;
    }

    protected function convertToXml($array, $rootElement = '<txn/>', $xml = null)
    {
        $_xml = $xml;

        // If there is no Root Element then insert root
        if ($_xml === null) {
            $_xml = new \SimpleXMLElement($rootElement !== null ? $rootElement : '<txn/>');
        }

        // Visit all key value pair
        foreach ($array as $k => $v) {
            // If there is nested array then
            if (is_array($v)) {
                $this->convertToXml($v, $k, $_xml->addChild($k));
            } else {
                $v = htmlspecialchars($v);
                // Simply add child element.
                $_xml->addChild($k, $v);
            }
        }

        return $_xml->asXML();
    }
}
