<?php
/**
 * 
 * @category  Payment Integration
 * @package   Risecommerce_Elavon
 * @copyright 2024 Rise Core Technologies Pvt. Ltd.. (https://risecommerce.com/)
 */
namespace Risecommerce\Elavon\Model\Ui;

use Magento\Checkout\Model\ConfigProviderInterface;

class ConfigProvider implements ConfigProviderInterface
{
    const CODE = 'risecommerce_elavon_option';
    const CC_VAULT_CODE = 'risecommerce_elavon_option_cc_vault';

    /**
     * @var \Magento\Payment\Model\Config
     */
    protected $_paymentConfig;

    /**
     * @var \Risecommerce\Elavon\Helper\Data
     */
    protected $dataHelper;

    /**
     * @param \Magento\Payment\Model\Config $paymentConfig
     * @param \Risecommerce\Elavon\Helper\Data $customHelper
     */
    public function __construct(
        \Magento\Payment\Model\Config $paymentConfig,
        \Risecommerce\Elavon\Helper\Data $customHelper
    ) {
        $this->_paymentConfig = $paymentConfig;
        $this->dataHelper = $customHelper;
    }

    public function getCcMonths()
    {
        return $this->_paymentConfig->getMonths();
    }

    public function getCcYears()
    {
        $years = [];
        $years = $this->_paymentConfig->getYears();
        return $years;
    }

    protected function getCcAvailableCcTypes()
    {
        return $this->dataHelper->getCcAvailableCardTypes();
    }

    /**
     * @return boolean
     */
    protected function getEnableCaptcha()
    {
        return $this->dataHelper->getEnableCaptcha();
    }

    /** @return string */
    protected function getCaptchaSiteKey()
    {
        return $this->dataHelper->getCaptchaSiteKey();
    }

    /**
     * @inheritDoc
     */
    public function getConfig(): array
    {
        $config = [
            'payment' => [
                self::CODE => [
                    'availableCardTypes' => $this->getCcAvailableCcTypes(),
                    'ccMonths' => $this->getCcMonths(),
                    'ccYears' => $this->getCcYears(),
                    'ccVaultCode' => self::CC_VAULT_CODE,
                    'iscaptchaenable' => $this->getEnableCaptcha(),
                    'captchasitekey' => $this->getCaptchaSiteKey()
                ]
            ]
        ];

        return $config;
    }
}
