<?php
namespace Risecommerce\Elavon\Plugin;

use Magento\Vault\Api\PaymentTokenRepositoryInterface;
use Magento\Vault\Api\Data\PaymentTokenInterface;
use Magento\Framework\Message\ManagerInterface;
use Risecommerce\Elavon\Model\Request\Api as CustomApi;

class DeleteVaultToken
{
    /**
     * @var ManagerInterface
     */
    protected $messageManager;

    /**
     * @var CustomApi
     */
    protected $_requestApi;

    /**
     * DeleteStoredPaymentPlugin constructor
     *
     * @param ManagerInterface $messageManager
     * @param CustomApi $requestApi
     */
    public function __construct(
        ManagerInterface $messageManager,
        CustomApi $requestApi
    ) {
        $this->_requestApi = $requestApi;
        $this->messageManager = $messageManager;
    }

    /**
     * @param PaymentTokenRepositoryInterface $subject
     * @param PaymentTokenInterface $paymentToken
     * @return bool|null
     */
    public function beforeDelete(PaymentTokenRepositoryInterface $subject, PaymentTokenInterface $paymentToken)
    {
        try {
            $requestResponse = $this->_requestApi->deleteProfile($paymentToken->getGatewayToken());
            if (isset($requestResponse['ssl_result']) && ($requestResponse['ssl_result'] == 0)) {
                $this->messageManager->addSuccessMessage(__("Payment token has been removed successfully from payment gateway server."));
            } else {
                $this->messageManager->addWarning(__("Issue in deleting credit card in the payment gateways server."));
            }
        } catch (\Exception $e) {
            $this->messageManager->addError(__($e->getMessage()));
        }

        return null;
    }
}
