define([
    'Magento_Payment/js/view/payment/cc-form',
    'jquery',
    'Magento_Vault/js/view/payment/vault-enabler',
    'Magento_Checkout/js/action/place-order',
    'Magento_Checkout/js/model/full-screen-loader',
    'Magento_Checkout/js/model/payment/additional-validators',
    'Magento_Payment/js/model/credit-card-validation/validator'
], function (
    Component,
    $,
    VaultEnabler
) {
    'use strict';

    var paymentConfig = window.checkoutConfig.payment.risecommerce_elavon_option;
    return Component.extend({
        defaults: {
            template: 'Risecommerce_Elavon/payment/elavon',
            cToken: null,
            captchaLoaded: false,
            generateC: 0
        },

        initObservable: function () {
            this._super();
            var self = this;
            if (this.isCaptchaEnabled() == 1) {
                var verifyCallback = function(response) {
                    self.cToken = response;
                };
                var includeCaptchaHtml = function () {
                    if (!jQuery('.rc-google-captcha-elavon')[0]) {
                        setTimeout(function(){includeCaptchaHtml();}, 500);
                        return;
                    }
                    if (!self.captchaLoaded) {
                        grecaptcha.render('rc-google-captcha-elavon', {
                          'sitekey' : self.cSiteKey(),
                           'callback' : verifyCallback,
                        });
                        self.captchaLoaded = true;
                    }
                };
                includeCaptchaHtml();
            }

            return this;
        },

        /**
         * @returns {exports.initialize}
         */
        initialize: function () {
            this._super();
            this.vaultEnabler = new VaultEnabler();
            this.vaultEnabler.setPaymentCode(this.getVaultCode());

            return this;
        },

        getData: function() {
            var data = this._super();
            this.vaultEnabler.visitAdditionalData(data);
            $.extend(
                true,
                data,
                {
                    'additional_data': {
                        'captcha_string': this.cToken
                    }
                }
            );

            return data;
        },

        isCaptchaEnabled: function() {
            return paymentConfig.iscaptchaenable;
        },

        cSiteKey: function() {
            //console.log('KEY = '+paymentConfig.captchasitekey);
            return paymentConfig.captchasitekey;
        },

        getCode: function() {
            return 'risecommerce_elavon_option';
        },

        /**
         * @returns {String}
         */
        getVaultCode: function () {
            return paymentConfig.ccVaultCode;
        },

        isActive: function() {
            return true;
        },

        /**
         * @returns {Bool}
         */
        isVaultEnabled: function () {
            return this.vaultEnabler.isVaultEnabled();
        },

        validate: function() {
            var $form = $('#' + this.getCode() + '-form');
            return $form.validation() && $form.validation('isValid');
        },

        getCcMonths: function() {
            return paymentConfig.ccMonths;
        },

        getCcYears: function() {
            return paymentConfig.ccYears;
        },

        getCcAvailableTypes: function() {
            return paymentConfig.availableCardTypes;
        },

        getCvvImageUrl: function () {
            return window.checkoutConfig.payment.ccform.cvvImageUrl[this.getCode()][0];
        },

        placeOrder: function (data, event) {
            var self = this;
            if (self.isCaptchaEnabled() != 3) {
               return self._super(data, event);
            } else {
                if (self.generateC == 1) {
                     self.generateC = 0;
                    return self._super(data, event);
                } else {
                    grecaptcha.ready(function() {
                        grecaptcha.execute(self.cSiteKey(), {action: 'submit'}).then(function(token) {
                            self.cToken = token;
                            self.generateOrder();
                            //return self._super(data, event);
                        });
                    });
                }
            }
        },

        generateOrder: function() {
            this.generateC = 1;
            this.placeOrder();
        }
    });
});
